import { ArrayWrapper } from './array';
import { MapWrapper } from './map';
import { ObjectWrapper } from './object';
import { SetWrapper } from './set';
export declare type Lifted<T> = undefined extends T ? never : null extends T ? never : T extends Wrapper<infer W> ? LiftedValue<W> : LiftedValue<T>;
declare type AtomicObject = Function | Promise<any> | Date | RegExp | Boolean | Number | String;
declare type LiftedValue<T> = null extends T ? never : undefined extends T ? never : T extends AtomicObject ? T : T extends ReadonlyArray<infer E> ? ArrayWrapper<E> : T extends ReadonlyMap<infer K, infer V> ? MapWrapper<K, V> : T extends ReadonlySet<infer E> ? SetWrapper<E> : T extends object ? ObjectWrapper<T> : never;
interface Lift {
    <T>(obj: ArrayWrapper<T>): ArrayWrapper<T>;
    <T extends object>(obj: ObjectWrapper<T>): ObjectWrapper<T>;
    <K, V>(obj: MapWrapper<K, V>): MapWrapper<K, V>;
    <T>(obj: SetWrapper<T>): SetWrapper<T>;
    /** lift won't wrap primitives */
    <T extends AtomicObject>(obj: T): T;
    /** Wraps an Array to provide a richer API. Unwrap with .value() **/
    <T>(obj: ReadonlyArray<T>): ArrayWrapper<T>;
    /** Wraps a Set to provide a richer API. Unwrap with .value() **/
    <T>(obj: ReadonlySet<T>): SetWrapper<T>;
    /** Wraps a Map to provide a richer API. Unwrap with .value() **/
    <K, V>(obj: ReadonlyMap<K, V>): MapWrapper<K, V>;
    /** Wraps an Object to provide a richer API. Unwrap with .value() **/
    <T extends object>(obj: T): ObjectWrapper<T>;
}
export declare const lift: Lift;
export declare function getValue<A>(input: A | Wrapper<A>): A;
export interface Wrapper<T> {
    value(): T;
}
export declare function pipe<T, R>(this: Wrapper<T>, func: (object: T) => R): Lifted<R>;
export {};
